/*	comms_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "comms_isrs.h"

/* SCI0 ISR handles all interrupts for SCI0 by reading the flag registers and acting appropriately. */
void SCI0ISR(void)
{
	/* Read the flags register */
	//unsigned char flags = SCI0SR1;
	
	/* From left most/most significant to right most/least significant.
	 * 
	 *     Abrev  Description					  Actions
	 * 1 = TRDE - Transmit Data Register Empty	- load with more data if there is any
	 * 1 = TC   - Transmit Complete				- turn flow back on
	 * 1 = RDRF - Receive Data Register Full	- Store data in buffer until packet complete.
	 * 1 = IDLE - Idle Line						- transmit datalogging if available
	 * 1 = OR   - Overrun						- discard partial data received and return error packet (code for overrun error) if configured to
	 * 1 = NF   - Noise Flag					- accept data, increment counter, send error packet if configured to
	 * 1 = FE   - Framing Error					- discard partial data received and return error packet (code for framing error) if configured to
	 * 1 = PF   - Parity Error					- discard data sent and return negative ack (code for bad parity) or error packet (same code) if configured to, increment counter
	 */

	// nested interrupts here? (reenable isrs before finishing to allow the engine to run best.)
	
	//if(material to send && character buffer open){
	//	send char and wait till next char slot is free.
	//}
	
	// loop sending chars from a buffer of some sort as available/time permits.	
}

// TODO Some of this should possibly be moved to the ISR, this is just a demo/experimental work for
// TODO getting data out in HR form for now. Soon it will be gutted and the serial spec implemented.

/* Send string over serial */
void sendSt(char *str, unsigned char length){
	int i;
	for (i=0;i<length;i++){
		// poll trde flag and send a byte when ready
		while(!(SCI0SR1 & 0x80)){/* Wait for ever until able to send then move on */}
		SCI0DRL = str[i];
	}
}

/* Send unsigned short over serial */
void sendUC(unsigned char value)
{
	char str[3];		// ascii number holder
	itoaUC(value, str);	// convert from unsigned char to ascii
	sendSt(str, 3);		// send the string on the serial line
}

/* Send unsigned short over serial */
void sendUS(unsigned short value)
{
	char str[5];		// ascii number holder
	itoaUS(value, str);	// convert from unsigned short to ascii
	sendSt(str, 5);		// send the string on the serial line
}

/* Send unsigned short over serial */
void sendUL(unsigned long value)
{
	char str[10];		// ascii number holder
	itoaUL(value, str);	// convert from unsigned long to ascii
	sendSt(str, 10);	// send the string on the serial line
}

//	To initiate an SCI transmission:
//		1. Configure the SCI: (in init.c only)
//			a)	Select a baud rate. Write this value to the SCI baud registers (SCIBDH/L) to begin the baud
//				rate generator. Remember that the baud rate generator is disabled when the baud rate is zero.
//				Writing to the SCIBDH has no effect without also writing to SCIBDL.
//			b)	Write to SCICR1 to configure word length, parity, and other configuration bits
//				(LOOPS,RSRC,M,WAKE,ILT,PE,PT).
//			c)	Enable the transmitter, interrupts, receive, and wake up as required, by writing to the SCICR2
//				register bits (TIE,TCIE,RIE,ILIE,TE,RE,RWU,SBK). A preamble or idle character will now
//				be shifted out of the transmitter shift register.
//		2. Transmit Procedure for Each Byte: (here)
//			a)	Poll the TDRE flag by reading the SCISR1 or responding to the TDRE interrupt. Keep in mind
//				that the TDRE bit resets to one.
//			b)	If the TDRE flag is set, write the data to be transmitted to SCIDRH/L, where the ninth bit is
//				written to the T8 bit in SCIDRH if the SCI is in 9-bit data format. A new transmission will not
//				result until the TDRE flag has been cleared.
//		3. Repeat step 2 for each subsequent transmission.
