/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "main.h"

void adjustPWM(void)
{
	PWMDTY0 = ATD0DR0 >> 2; // scale raw adc to a duty
	PWMDTY1 = ATD0DR1 >> 2; // scale raw adc to a duty
	PWMDTY2 = ATD0DR2 >> 2; // scale raw adc to a duty
	PWMDTY3 = ATD0DR3 >> 2; // scale raw adc to a duty
	PWMDTY4 = ATD0DR4 >> 2; // scale raw adc to a duty
	PWMDTY5 = ATD0DR5 >> 2; // scale raw adc to a duty
	PWMDTY6 = ATD0DR6 >> 2; // scale raw adc to a duty
	//PWMDTY7 = ATD0DR7 >> 2; // scale raw adc to a duty (user led instead)
}

/* Based on the latest ADC readings, determine pulsewidth, dwell, and advance for all channels */
void calculateFuelAndIgnition(void)
{
	/* Set up the banks and offsets for us to use for input and output */
	/* Set up the banks for input and output */
	unsigned char localInputBank = outputBank;
	unsigned char localOutputBank = inputBank;
	/* Set up local output bank offsets */
	unsigned char localInjectionOffset;
	unsigned char localIgnitionOffset;
	if(localOutputBank){
		localInjectionOffset = 0;
		localIgnitionOffset = 0;
	}else{
		localInjectionOffset = INJECTION_CHANNELS;
		localIgnitionOffset = IGNITION_CHANNELS;
	}

	/* "Calculate" the nominal total pulse width before per channel corrections */
	totalPulseWidth = (ego[localInputBank] << 6) + (map[localInputBank] >> 4);

	/* "Calculate" the individual fuel pulse widths */
	unsigned char channel;
	for(channel = 0; channel < INJECTION_CHANNELS; channel++){
		injectorMainPulseWidths[channel + localInjectionOffset] = totalPulseWidth;
	}

	// TODO x 6 main, x 6 staged, x 6 flags for staged if(coreSettingsA & STAGED_ON){}
	
	/* Set the staged status on or off (for now based on changeable settings) */
	if(coreSettingsA & STAGED_ON){
		coreStatusA |= STAGED_REQUIRED;
	}else{
		coreStatusA &= ~STAGED_REQUIRED;
	}
	
	// temporary ign tests
	unsigned short intendedAdvance = spareADC7[localInputBank] << 6;
	unsigned short intendedDwell = intendedAdvance >> 1;

	short c;
	for(c=0;c<IGNITION_CHANNELS;c++){
		ignitionAdvances[IGNITION_CHANNELS + localIgnitionOffset] = intendedAdvance;
	}
	currentDwell[localOutputBank] = intendedDwell;
	
	
	
	
	
//	unsigned short minPeriod = ignitionMinimumDwell << 1;
	//	if(intendedDwell < ignitionMinimumDwell){
//		dwellLength = ignitionMinimumDwell;
//	}else{
//		dwellLength = intendedDwell;
//	}
//	if(intendedPeriod < minPeriod){
//		dwellPeriod = minPeriod;
//	}else{
//		dwellPeriod = intendedPeriod;
//	}
//	PITLD0 = dwellPeriod;
	
	/* Calculate the fuel advances */
	// TODO x 6
	// just use one for all for now
	totalAngleAfterReferenceInjection = (tps[localInputBank] << 6);
	
	/* Calculate the dwell period */
	// TODO x 1
	
	/* Calculate the ignition advances */
	//TODO x 12
}

int  main(){
	// Set everything up.
	init();
		
	// Run forever repeating.
	while(TRUE){
	//	unsigned short start = realTimeCounterMillis;
		/* If ADCs require forced sampling, sample now */
		if(coreStatusA & FORCE_READING){
			/* Atomic block to ensure a full set of readings are taken together */
			ATOMIC_START();
			
			/* Check to ensure that a reading wasn't take before we entered a non interruptable state */
			if(coreStatusA & FORCE_READING){ // do we still need to do this

				/* Read ADCs into the correct bank */			
				iat[inputBank] = ATD0DR0;
				clt[inputBank] = ATD0DR1;
				tps[inputBank] = ATD0DR2;
				ego[inputBank] = ATD0DR3;
				map[inputBank] = ATD0DR4;
				spareADC5[inputBank] = ATD0DR5;
				spareADC6[inputBank] = ATD0DR6;
				spareADC7[inputBank] = ATD0DR7;

				/* Set flag to say calc required */
				coreStatusA |= CALC_FUEL_IGN;

				/* Clear force reading flag */
				coreStatusA &= ~FORCE_READING;
			}

			ATOMIC_END();
		}

		/* If required, do main fuel and ignition calcs first */
		if(coreStatusA & CALC_FUEL_IGN){
			/* Atomic block to ensure that we don't clear the flag for the next data set when things are tight */
			ATOMIC_START();
			
			/* Switch input bank so that we have a stable set of variables to read from */
			if(inputBank){
				inputBank = 0;
			}else{
				inputBank = 1;
			}
			
			/* Clear the calc required flag */
			coreStatusA &= ~CALC_FUEL_IGN;

			ATOMIC_END();
			
			/* Perform the calculations TODO possibly move this to the software interrupt if it makes sense to do so */
			calculateFuelAndIgnition();
			
			/* Atomic block to ensure that outputBank and outputBank Offsets match */
			ATOMIC_START();
			
			/* Switch output bank to latest */
			if(outputBank){
				outputBank = 0;
				outputBankInjectionOffset = 0;
				outputBankIgnitionOffset = 0;
			}else{
				outputBank = 1;
				outputBankInjectionOffset = INJECTION_CHANNELS;
				outputBankIgnitionOffset = IGNITION_CHANNELS;
			}
			
			ATOMIC_END();
			
			/* Allow outputs to be read from */
			coreStatusA |= CALCS_VALID;
		}

		// Measure run time :
	//	unsigned short stop = realTimeCounterMillis; 
	//	unsigned short runtime = stop - start;
		
		unsigned short secondsBuffer = realTimeCounterSeconds;
		
		if(lastSeconds != secondsBuffer){
			char endEntire[8] = {'[','E','n','d',']','X',0x0A,0x0D};

			/* Comment out and add in vars as you wish.
			 * 
			 * The only requirements are that you put the length of your string in correctly
			 * and use the correct send function for the data type required. Using the Long
			 * one for all types should work, but would be slow, so there are 3 types for now.
			 * 
			 * Unsigned Char
			 * Unsigned Short
			 * Unsigned Long
			 * 
			 * Signed numbers can not be displayed correctly, though this could be added fairly easily.
			 */
			
			// Serial experimentation
			sendSt("@[FreeEMS Logging Data]", 24);

			sendSt("Seconds = ", 10);
			sendUS(realTimeCounterSeconds);
			sendSt(", TotalPW = ", 10);
			sendUS(totalPulseWidth);
			sendSt(", RPM = ", 8);
			sendUS(rpm[outputBank]);
			sendSt(", TicksPerRev = ", 16);
			sendUL(engineCyclePeriod); // yes, this var is being misused and is per rev, not cycle.
			sendSt(", PPPSP = ", 10);
			sendUS(primaryPulsesPerSecondaryPulseBuffer);
			sendSt(", delayFromRefInj = ", 20);
			sendUS(totalAngleAfterReferenceInjection);
			sendSt(", LostCrankSync = ", 18);
			sendUC(lostCrankSyncCounter);
			
			// dwell and advance currently match fuel for comparative purposes...
//			sendSt(", delayFromRefIgn = ", 20);
//			sendUS(totalAngleAfterReferenceIgnition);
			// currently too fast to care about and padded with a sleep
//			sendSt(", MainLoopTimeMS = ", 19);
//			sendUS(runtime);
			// not likely to occur on the new test rig, but good to have in-car
//			sendSt(", LowVolts = ", 13);
//			sendUC(lowVoltISRCounter);
			// not currently used.
//			sendSt(", LostCamSync = ", 16);
//			sendUC(lostCamSyncCounter);

			sendSt(endEntire, 8);

			// mechanism to ensure we send once per second without doing it in the RTC section.
			lastSeconds = secondsBuffer;
		}
		
		// Do not remove until there is sufficient code in here to make the main loop take a while
		sleep(5); // Ignoring this causes interrupts to be locked out far far too often.
		
		// on once per cycle for J0 (tacho)
		PORTJ ^= 0x01;

		// PWM experimentation
		adjustPWM();
	}
}
