/*	misc_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "misc_isrs.h"

/* Unimplemented interrupt service routine */
void UISR(void){
	/* Increment the unimplemnted ISR execution counter */
	UISRCounter++;
}

void PortPISR(void){
	/* Clear all port P flags (we only want one at a time) */
	PIFP = ONES;
	/* Increment the unimplemnted ISR execution counter */
	UISRCounter++;
}			/* Port P interrupt service routine */

/* Port J interrupt service routine */
void PortJISR(void){
	/* Clear all port H flags (we only want one at a time) */
	PIFJ = ONES;
	/* Increment the unimplemnted ISR execution counter */
	UISRCounter++;
}

/* Port H interrupt service routine */
void PortHISR(void)
{
	// read the interrupt flags to a variable
	unsigned char portHFlags = PIFH;
	portHFlags &= 0xF8; // mask out the other bits
	
	/* Clear all port H flags (we only want one at a time) */
	PIFH = ONES;

	// Toggle a LED so we can see if the code ran
	PORTS ^= 0x02;
	
	// debounce
	if(portHDebounce == 0){
		portHDebounce = 2;
	}else{
		return;
	}

	// find out which pin triggered it, clear the flag, perform the action.
	switch(portHFlags)
	{
	case 0x80 :	// Increment cylinder count and set port count appropriately.
		switch (combustionEventsPerEngineCycle) {
			case 1 :
				combustionEventsPerEngineCycle = 2;
				ports = 2;
				break;
			case 2 :
				combustionEventsPerEngineCycle = 3;
				ports = 3;
				break;
			case 3 :
				combustionEventsPerEngineCycle = 4;
				ports = 4;
				break;
			case 4 :
				combustionEventsPerEngineCycle = 5;
				ports = 5;
				break;
			case 5 :
				combustionEventsPerEngineCycle = 6;
				ports = 6;
				break;
			case 6 :
				combustionEventsPerEngineCycle = 8;
				ports = 4;
				break;
			case 8 :
				combustionEventsPerEngineCycle = 10;
				ports = 5;
				break;
			case 10 :
				combustionEventsPerEngineCycle = 12;
				ports = 6;
				break;
			case 12 :
				combustionEventsPerEngineCycle = 1;
				ports = 1;
				break;
		}
		break;
	case 0x40 : // Injection output enable/disable
		if(coreSettingsA & FUEL_CUT){
			coreSettingsA &= ~FUEL_CUT;
		}else{
			coreSettingsA |= FUEL_CUT;
		}
		break;
	case 0x20 : // Ignition output enable/disable
		if(coreSettingsA & HARD_SPARK_CUT){
			coreSettingsA &= ~HARD_SPARK_CUT;
		}else{
			coreSettingsA |= HARD_SPARK_CUT;
		}
		break;
	case 0x10 : // Staged injection enable/disable
		if(coreSettingsA & STAGED_ON){
			coreSettingsA &= ~STAGED_ON;
		}else{
			coreSettingsA |= STAGED_ON;
		}
		break;
	case 0x08 : // Staged injection start sched/fixed
		if(coreSettingsA & STAGED_START){
			coreSettingsA &= ~STAGED_START;
		}else{
			coreSettingsA |= STAGED_START;
		}
		break;
	case 0x04 : // Staged injection end sched/fixed
		if(coreSettingsA & STAGED_END){
			coreSettingsA &= ~STAGED_END;
		}else{
			coreSettingsA |= STAGED_END;
		}
		break;
	case 0x02 : // free input
		break;
	case 0x01 : // free input
		break;
	default : // Two or more pressed, nothing to do except wait for another button press
		break;
	}	
}

/* IRQ/PE1 interrupt service routine */
void IRQISR(void){
	/* Clear the flag */
	// ?? TODO
	
	/* Increment the unimplemnted ISR execution counter */
	UISRCounter++;
}

/* XIRQ/PE0 interrupt service routine */
void XIRQISR(void){
	/* Clear the flag */
	// ?? TODO
	
	/* Increment the unimplemnted ISR execution counter */
	UISRCounter++;
}

/* Count how often our voltage goes too low without resetting */
void LowVoltageISR(void){
	/* Clear the flag */
	VREGCTRL |= 0x01;

	/* Increment the counter */
	lowVoltISRCounter++;
}
