/*	utils.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "utils.h"

/* Loop repeatedly for a milli second. */
void sleep(unsigned short ms)
{
	unsigned short j, k, loop;
	loop = ms * 2;

	for(j=0;j<loop;j++)
		for(k=0;k<2857;k++);
}

/* Read two ADC inputs and use them to sleep anywhere from 0.001ms to 1 second */
void sleep2()
{
	// Initial sample for external loop
	unsigned short adc3raw = ATD0DR3 + 1; // O2 pot
	unsigned short period1 = 2048 / adc3raw;

	unsigned short j, k;

	// external loop
	for(j=0;j<period1;j++){
		// Sample ADC for internal loop as we need it
		unsigned short adc2raw = ATD0DR2 + 1; // TPS pot
		unsigned short period2 = 2048 / adc2raw;
		
		// internal loop
		for(k=0;k<period2;k++);
		
		// Re-sample ADC for external loop as we need it
		adc3raw = ATD0DR3 + 1;
		period1 = 2048 / adc3raw;
	}
}

// TODO UGLY as sin, duplication here, not sure about types etc, will fix later if possible.

// It seems to me that a LONG would do for all types, but would produce a long or variable string which is not ideal.
// It would also be horribly slow being non native. Perhaps this style of separate implementation is best?

/* Generate the ascii representation of a char */
void itoaUC(unsigned char n, char *s){
    int j;
    for(j=0;j<3;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[2 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
}

/* Generate the ascii representation of a short */
void itoaUS(unsigned short n, char *s)
{
    int j;
    for(j=0;j<5;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[4 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
} 

/* Generate the ascii representation of a long */
void itoaUL(unsigned long n, char *s){
    int j;
    for(j=0;j<10;j++){				/* Loop through all places and fill with zeros if nothing else	*/
   		s[9 - j] = n % 10 + '0';	/* Place the least significant digit at the end of the array	*/
   		n /= 10;					/* Shift to the right one decimal place							*/
    }
}
